import { ChangeDetectorRef, Component, EventEmitter, Input, OnInit, } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { Bu1Model, MyBu1Model } from 'src/app/shared/model/bu1.model';
import { Bu1Service } from 'src/app/shared/services/bu1.service';
import { FileService } from 'src/app/shared/services/file.service';

@Component({
  selector: 'app-department-register',
  templateUrl: './department-register.component.html',
  styleUrls: ['./department-register.component.scss']
})
export class DepartmentRegisterComponent implements OnInit {
  currentPage = 1
  pageSize = 10
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  bu1List: { check: boolean, data: Bu1Model }[] = []
  bu1ListLoading = false
  bu1: Bu1Model = new MyBu1Model()
  search = ""

  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';

  currentModel: 'add' | 'edit' | 'delete' | 'deleteGroup' = "add"

  numDataListChecked = 0
  isDataListChecked = false
  isDataListCheckedAll = false
  constructor(private bu1Service: Bu1Service,
    private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private fileService: FileService
  ) { }
  ngOnInit(): void {
    this.getBu1List()
  }
  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }

  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.bu1ListLoading = true
    this.fileService.uploadExcel(formData, 'mbu1').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getBu1List()
        } else {
          this.showAlert(response.message, 'error')
          this.bu1ListLoading = false
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.bu1ListLoading = false
      }
    })
  }

  downloadFile() {
    const fileName = 'IMPORT_BU.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }


  getBu1List() {
    this.bu1ListLoading = true
    this.bu1Service.getList().subscribe({
      next: response => {
        this.bu1List = response.map(x => ({ check: false, data: new MyBu1Model(x) }))
        this.isDataListCheckedAll = false
        this.dataListCheckAll()
        this.bu1ListLoading = false
        this.searchChange()
        this.cdr.detectChanges()
      }, error: error => {
        this.bu1ListLoading = false
        this.cdr.detectChanges()
      }
    })
  }
  searchChange() {
    this.currentPage = 1
    this.page = Array.from({ length: Math.ceil(this.bu1ListFilter().length / 10) }, (_, i) => i + 1);
    this.dataListCheck()
  }
  bu1ListFilter() {
    return this.bu1List.filter(x => {
      const data = x.data
      return data.bu1id.toLowerCase().includes(this.search.toLowerCase()) ||
        data.tdesc.toLowerCase().includes(this.search.toLowerCase()) ||
        data.edesc.toLowerCase().includes(this.search.toLowerCase())
    })
  }
  selectBu1(bu1?: Bu1Model) {
    this.bu1 = new MyBu1Model(bu1)
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }
  addBu1() {
    this.bu1ListLoading = true
    this.bu1Service.post(this.bu1).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getBu1List()
        } else {
          this.showAlert(response.message, 'error')
          this.bu1ListLoading = false
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.bu1ListLoading = false
      }
    })
  }
  deleteBu1() {
    this.bu1ListLoading = true
    const body = this.bu1List.filter(x => x.check).map(x => new MyBu1Model(x.data))
    this.bu1Service.delete(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getBu1List()
        } else {
          this.showAlert(response.message, 'error')
          this.bu1ListLoading = false
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.bu1ListLoading = false
      }
    })
  }
  dataListCheckAll() {
    const selectAll = this.isDataListCheckedAll;
    this.bu1ListFilter().forEach(x => x.check = selectAll);
    this.dataListCheck();
  }

  dataListCheck() {
    const dataCheck = this.bu1ListFilter();
    this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
    this.numDataListChecked = this.bu1List.filter(x => x.check).length
    this.isDataListChecked = Boolean(this.numDataListChecked)
  }

}
